// vim:set tabstop=4 cindent:

// Author: Geert-Jan Giezeman <geert@cs.uu.nl>
// Copyright: Utrecht University, 2011

#include "jump_detection.hpp"
#include "geometry.hpp"
#include <math.h>

using std::vector;
using UuIcsC3d::FrameData;
using UuIcsC3d::LabeledSkeleton;
using std::string;


namespace {
	struct PositionRecord {
		bool pt_valid;
		UuIcsGeom::Point3d pt; // Point
		bool d_valid;
		UuIcsGeom::Vector3d d; // pt - prev->pt
		PositionRecord *next, *prev;
	};

	void report_jumps(std::ostream &os,
		vector<FrameData> &frame_data,
		int marker, string const &label_name)
	{
		using namespace UuIcsGeom;
		double const MaxSpeed = 35.0;
		double const MaxA = 20.0;
		// make circular buffer
		size_t const PosCount = 3;
		PositionRecord cache[PosCount];
		for (size_t i=0; i!=PosCount; ++i) {
			cache[i].d_valid = false;
			cache[i].pt_valid = false;
		}
		for (size_t i=0; i!=PosCount-1; ++i) {
			cache[i].next = cache+(i+1);
			cache[i+1].prev = cache+i;
		}
		cache[0].prev = cache+(PosCount-1);
		cache[PosCount-1].next = cache;
		PositionRecord *cpos = cache+0;

		for (size_t f=0; f<frame_data.size(); ++f, cpos=cpos->next) {
			cpos->pt_valid = frame_data[f].points[marker].is_valid();
			if (cpos->pt_valid) {
				cpos->pt_valid = true;
				cpos->pt = Point3d(frame_data[f].points[marker].coordinates());
				cpos->d_valid = cpos->prev->pt_valid;
				if (cpos->d_valid) {
					cpos->d = cpos->pt - cpos->prev->pt;
					if (cpos->prev->d_valid) {
						Vector3d avec = cpos->d - cpos->prev->d;
						double a = inprod(avec, avec);
						if (a>MaxA*MaxA) {
							os<<marker<<' '<<label_name<<' '
								<<f<<' '<<sqrt(a)<<"\n";
						}
					} else {
						double d = inprod(cpos->d, cpos->d);
						if (d>MaxSpeed*MaxSpeed) {
							os<<marker<<' '<<label_name<<' '
								<<f<<' '<<sqrt(d)<<" (speed)\n";
						}
					}
				} 
			} else {
				cpos->d_valid = false;
			}
		}
	}
}

void report_jumps(
		std::ostream &os,
		vector<FrameData> &frame_data,
		vector<LabeledSkeleton> const &subjects, 
		vector<UuIcsC3d::SpacePaddedString> const &labels)
{
	os<<"Maximal jumps for each label (label, to_frame jump)\n";
	for (size_t s=0; s!= subjects.size(); ++s) {
		vector< vector<int> > const &point_index = subjects[s].point_index();
		for (size_t g=0; g!= point_index.size(); ++g) {
			vector<int> const &idx=point_index[g];
			for (size_t m=0; m!=idx.size(); ++m) {
				if (idx[m]>=0)
					report_jumps(os, frame_data, idx[m], labels[idx[m]].stripped());
			}
		}
	}
}
