// vim:set tabstop=4 cindent:

// Author: Geert-Jan Giezeman <geert@cs.uu.nl>
// Copyright: Utrecht University, 2011

#include "jump_detection.hpp"
#include "uuc3d.hpp"
#include <iostream>
#include <fstream>
#include "subjects.hpp"


using std::string;
using std::vector;
using std::cerr;
using UuIcsC3d::SpacePaddedString;
using UuIcsC3d::FrameData;
using UuIcsC3d::LabeledSkeleton;

UuIcsC3d::SkeletonInfo const &get_uu_skeleton_info();

// TODO: Use CGAL::Min_annulus_d<Optimisation_d_traits_3> for computing
// center of rotation for joints. Do this relative to frame of e.g. knee.

/*
void check_skeleton_info()
{
	// visual consistency check for skeleton
	using std::cout;
	SkeletonInfo const &skinf = get_uu_skeleton_info();
	cout<<skinf.group_count()<<" groups.\n";
	for (int i=0; i<skinf.group_count(); ++i) {
		cout<<"Group "<<skinf.group_name(i)<<"\n";
		ConstNameRange members(skinf.group_members(i));
		for (std::string const *cur =members.begin(); cur !=members.end(); ++cur) {
			cout<<"\t"<<*cur<<"\n";
		}
		vector<int> cnx = skinf.group_connections(i);
		cout<<"  connections: \n";
		for (unsigned j=0; j<cnx.size(); ++j) {
			cout<<"\t"<<skinf.group_name(cnx[j])<<"\n";
		}
	}
}

void left_arm_model(vector<FrameData> const &frame_data, LabeledSkeleton const &subjects)
{
	int const LeftElbow = 10; 
	int const LeftUparm = 9;
	int n1 = subjects.point_index()[LeftUparm].size();
	int n2 = subjects.point_index()[LeftElbow].size();
	int n = n1+n2;
	vector<int> point_index(n);
	for (int i=0; i<n2; ++i)
		point_index[i] = subjects.point_index()[LeftElbow][i];
	for (int i=0; i<n1; ++i)
		point_index[n2+i] = subjects.point_index()[LeftUparm][i];
	vector<Point3d> points(n);
	vector<vector<Point3d> > objects;
	for (size_t i=0; i<frame_data.size(); ++i) {
		FrameData const &cur_frame = frame_data[i];
		bool all_valid = true;
		for (int j=0; j!=n; ++j) {
			UuIcsC3d::DataPoint3d pt = cur_frame.points[point_index[j]];
			if (!pt.is_valid()) {
				all_valid = false;
				break;
			}
			points[j] = Point3d(pt.coordinates);
		}
		if (!all_valid)
			continue;
		objects.push_back(points);
	}
	if (objects.size() >0) {
		vector<Vector3d> model = make_model(objects);
	}
}
*/


void do_work(string infile, string outfile)
{
//  check_skeleton_info();
//  check_basis();
    UuIcsC3d::C3dFileInfo fi(infile);
    int ppf = fi.points_per_frame(); 
    if (ppf <= 0)
        return;
    int fc = fi.frame_count();
    vector<SpacePaddedString> labels(fi.point_labels());
    vector<LabeledSkeleton> subjects;
    vector<unsigned> no_label = fill_skeletons(
			subjects, labels, get_uu_skeleton_info());
    for (int i=labels.size(); i<ppf; ++i) {
        labels.push_back(SpacePaddedString("<no label>"));
    }
    //LabellingStatistics stats(ppf, subjects);
    std::auto_ptr<UuIcsC3d::C3dFile> filep(fi.open());
    vector<FrameData> frame_data(fc);
    for (int i=0; i<fc; ++i) {
        filep->get_frame_data(frame_data[i], i);
    }
    filep.reset();

    // Check model approach
//  left_arm_model(frame_data, subjects[0]);
	std::ofstream ofs(outfile.c_str());
	if (!ofs) {
		throw UuIcsC3d::OpenError(outfile);
	}
	report_jumps(ofs, frame_data, subjects, labels);
}

//void check_matrix();
//void check_fitting();

char *progname=0;

void print_usage(std::ostream &os)
{
    os<<"Usage:\n";
    os<<progname<<" input_c3d logfile\n";
}

int main(int argc, char* argv[])
{
//  check_matrix();
//  check_fitting();
//  return 0;
    progname = argv[0];
    if (argc<3) {
        print_usage(cerr);
        return 1;
    }
    try {
        do_work(argv[1], argv[2]);
    } catch (UuIcsC3d::OpenError const &err) {
        cerr<<"An error occurred while opening file "<<err.filename();
        return 2;
	}
    return 0;
}
