#ifndef FIT378DHEMMWKQ946SGSH3
#define FIT378DHEMMWKQ946SGSH3

// vim:set tabstop=4 cindent:

// Author: Geert-Jan Giezeman <geert@cs.uu.nl>
// Copyright: Utrecht University, 2011

#include "geometry.hpp"
#include <vector>

//! \file
//! \brief Fitting of an instance to a model
//!
//! An instance is a vector of Point3d points.
//! A model is a vector of Vector3d points.
//! Related instances and models should always have the same number of points.
//! The order of the points is important.

//! \brief Result of fitting an instance to a model by the funcion \ref ::fit.
//!
//! The result contains a distance, a description of the transformation
//! that is applied to attain this distance and the transformed object.
struct FittingResult {
	//! The distance (sum of squared distances from transformed point to model point)
	double d;
	//! The translation to be applied to the instance (before the rotation)
	UuIcsGeom::Vector3d tr;
	//! The rotation to be applied to the instance (after the translation
	UuIcsGeom::Matrix3d rot;
	// The instance after applying the translation and then the rotation .
	std::vector<UuIcsGeom::Vector3d> transformed_object;
};

//! \brief Finds a best fit of the instance to the model.
//!
//! The instance is translated and rotated such that the sum of squared distances
//! from instance to model points is minimal.
FittingResult fit(std::vector<UuIcsGeom::Vector3d> const &model,
                std::vector<UuIcsGeom::Point3d> const &instance);

//! \brief Construct a model from a number of instances
//!
//! All instances should contain the same number of points.
//! The model is a kind of average of the instances.
std::vector<UuIcsGeom::Vector3d>  make_model(
                std::vector< std::vector<UuIcsGeom::Point3d> > const &instances);

#endif // FIT378DHEMMWKQ946SGSH3
